@@ -28,7 +28,7 @@ test_that("anomauth", {
   sm$pars$indvarying<- FALSE
   sf=ctStanFit(ctDeintervalise(ctWideToLong(AnomAuth,Tpoints = AnomAuthmodel$Tpoints,n.manifest = 2)),
     ctstanmodel = sm, optimize=TRUE,verbose=0,savescores = FALSE,cores=2,nopriors=TRUE,
-    optimcontrol=list(finishsamples=100),plot=FALSE)
+    optimcontrol=list(finishsamples=100))
   expect_equal(23415.929,-2*sf$stanfit$optimfit$value,tolerance=.01)
   anoms=summary(sf)
   expect_equivalent(.036,anoms$popmeans['manifestmeans_Y1','sd'],tolerance=.004)
